@@ -11,7 +11,7 @@ use Coro::Channel;
 use Coro::ProcessPool::Process;
 use Coro::ProcessPool::Util qw(cpu_count);
 
-our $VERSION = '0.15_02';
+our $VERSION = '0.16_01';
 
 if ($^O eq 'MSWin32') {
     die 'MSWin32 is not supported';
@@ -24,6 +24,7 @@ sub new {
         max_reqs  => $param{max_reqs}  || 0,
         num_procs => 0,
         procs     => Coro::Channel->new(),
+        pending   => {},
     }, $class;
 }
 
@@ -66,22 +67,12 @@ sub checkout_proc {
     if (!defined $timeout) {
         return $self->{procs}->get;
     } else {
-        my $cv = AnyEvent->condvar;
-        my $proc;
+        my $cv           = AnyEvent->condvar;
+        my $thread_timer = async { Coro::AnyEvent::sleep($timeout), $cv->send(0) };
+        my $thread_proc  = async { $cv->send($self->{procs}->get) };
+        my $proc         = $cv->recv;
 
-        my $thread_timer = async {
-            Coro::AnyEvent::sleep $timeout;
-            $cv->send(0);
-        };
-
-        my $thread_proc = async {
-            $proc = $self->{procs}->get;
-            $cv->send(1);
-        };
-
-        my $result = $cv->recv;
-
-        if ($result) {
+        if ($proc) {
             $thread_timer->cancel;
             return $proc;
         } else {
@@ -90,7 +81,7 @@ sub checkout_proc {
     }
 }
 
-sub process {
+sub start_task {
     my ($self, $f, $args, $timeout) = @_;
     defined $f || croak 'expected CODE ref or task class (string) to execute';
     $args ||= [];
@@ -99,6 +90,7 @@ sub process {
     my $proc = $self->checkout_proc($timeout);
 
     # Restart the process if the worker is exhausted
+    my $thread;
     if ($self->{max_reqs} > 0 && $proc->{processed} >= $self->{max_reqs}) {
         $self->kill_proc($proc);
         $proc = $self->start_proc;
@@ -107,14 +99,32 @@ sub process {
     # Send the task
     my $msgid = $proc->send($f, $args);
 
-    # Replace process in the pool as soon as result is ready on the connection
-    $proc->readable;
-    $self->{procs}->put($proc);
+    # Note which process is handling this task
+    $self->{pending}{$msgid} = $proc;
 
-    # Collect and return the result
+    # Replace process in the pool as soon as result is ready on the connection
+    async_pool {
+        my ($self, $proc) = @_;
+        $proc->readable;
+        $self->checkin_proc($proc);
+    } $self, $proc;
+
+    return $msgid;
+}
+
+sub collect_task {
+    my ($self, $msgid) = @_;
+    my $proc = $self->{pending}{$msgid} || croak 'msgid not found';
+    delete $self->{pending}{$msgid};
     return $proc->recv($msgid);
 }
 
+sub process {
+    my $self  = shift;
+    my $msgid = $self->start_task(@_);
+    return $self->collect_task($msgid);
+}
+
 sub map {
     my ($self, $f, @args) = @_;
     my @deferred = map { $self->defer($f, [$_]) } @args;
@@ -122,28 +132,41 @@ sub map {
 }
 
 sub defer {
-    my ($self, $f, $args) = @_;
-    my $arr = wantarray;
-    my $cv  = AnyEvent->condvar;
+    my $self  = shift;
+    my $arr   = wantarray;
+    my $cv    = AnyEvent->condvar;
+    my $msgid = $self->start_task(@_);
 
     async_pool {
-        my $arr = shift;
+        my ($self, $arr, $msgid) = @_;
 
         if ($arr) {
-            my @results = eval { process(@_) };
+            my @results = eval { $self->collect_task($msgid) };
             $cv->croak($@) if $@;
             $cv->send(@results);
         } else {
-            my $result = eval { process(@_) };
+            my $result = eval { $self->collect_task($msgid) };
             $cv->croak($@) if $@;
             $cv->send($result);
         }
-    } $arr, $self, $f, $args;
+    } $self, $arr, $msgid;
 
     return sub { $cv->recv };
 }
 
-sub DESTROY { $_[0]->shutdown }
+sub queue {
+    my ($self, $f, $args, $k) = @_;
+    my $deferred = $self->defer($f, $args);
+
+    async_pool {
+        my ($deferred, $k) = @_;
+        if (ref $k && ref $k eq 'CODE') {
+            $k->($deferred->());
+        } else {
+            $deferred->();
+        }
+    } $deferred, $k;
+}
 
 1;
 __END__
@@ -251,6 +274,30 @@ reference that, when called, returns the results of calling C<$f->(@$args)>.
     my $deferred = $pool->defer($coderef, [ $x, $y, $z ]);
     my $result   = $deferred->();
 
+=head2 queue($f, $args, $callback)
+
+Queues the execution of C<$f->(@$args)> and returns immediately. If
+C<$callback> is specified and is a code ref, it will be called with the result
+of the executed code once the task is processed. Note that the callback is not
+provided with any identifying information about the task being executed. That
+is the responsibility of the caller. It is also the caller's responsibility to
+coordinate any code that depends on the result, for example using a condition
+variable.
+
+    my $cv = AnyEvent->condvar;
+
+    sub make_callback {
+        my $n = shift;
+        return sub {
+            my $result = shift;
+            print "2 * $n = $result\n";
+            $cv->send;
+        }
+    }
+
+    $pool->queue($doubler_function, [21], make_callback(21));
+    $cv->recv;
+
 =head2 shutdown
 
 Shuts down all processes and resets state on the process pool. After calling
