@@ -4,43 +4,43 @@ use strict;
 use warnings;
 use Carp;
 
-use Coro qw(async cede);
-use Coro::Handle qw(unblock);
-use Coro::AnyEvent qw();
 use Config;
-use IPC::Open3 qw(open3);
-use POSIX qw(:sys_wait_h);
-use String::Escape qw(backslash);
-use Symbol qw(gensym);
-use Coro::ProcessPool::Mailbox;
-use Coro::ProcessPool::Util qw(encode decode $EOL);
+use Coro                        qw(async cede);
+use Coro::AnyEvent              qw();
+use Coro::Handle                qw(unblock);
+use Coro::ProcessPool::Mailbox  qw();
+use Coro::ProcessPool::Util     qw(encode decode $EOL);
+use IPC::Open3                  qw(open3);
+use POSIX                       qw(:sys_wait_h);
+use String::Escape              qw(backslash);
+use Symbol                      qw(gensym);
 
 if ($^O eq 'MSWin32') {
     die 'MSWin32 is not supported';
 }
 
-use fields qw(
-    pid
-    child_err
-    child_err_mon
-    processed
-    mailbox
-);
+sub new {
+    my ($class, %param) = @_;
+    return bless {
+        processed     => 0,
+        pid           => undef,
+        child_err     => undef,
+        child_err_mon => undef,
+        mailbox       => undef,
+    }, $class;
+}
 
 sub DESTROY {
     if ($_[0] && $_[0]->{pid}) {
-        $_[0]->terminate(1);
-        $_[0]->cleanup;
+        my $pid = $_[0]->{pid};
+        while ($pid > 0) {
+            $pid = waitpid($pid, WNOHANG);
+            Coro::AnyEvent::sleep(0.1)
+                if $pid > 0;
+        }
     }
 }
 
-sub new {
-    my ($class, %param) = @_;
-    my $self = fields::new($class);
-    $self->{processed} = 0;
-    return $self;
-}
-
 #-------------------------------------------------------------------------------
 # Return the full path to the perl binary used to launch the parent in order to
 # ensure that children are run on the same perl version.
@@ -80,7 +80,7 @@ sub spawn {
 
     $self->{child_err_mon} = async {
         while (my $line = $self->{child_err}->readline) {
-            warn "(WORKER) $line";
+            warn "(WORKER PID $self->{pid}) $line";
         }
     };
 
@@ -95,28 +95,17 @@ sub is_running {
 }
 
 sub terminate {
-    my ($self, $block) = @_;
-    my $pid = $self->{pid};
+    my $self = shift;
 
     if ($self->is_running) {
-        $self->{child_err}->close;
-
-        $self->{child_err_mon}->safe_cancel;
+        my $pid = $self->{pid};
+        $self->{mailbox}->shutdown;
         $self->{child_err_mon}->join;
 
-        if (kill(0, $pid)) {
-            warn("Error killing pid %d: %s", $pid, $!)
-                unless kill(9, $pid) || $!{ESRCH};
-        }
-
-        if ($block) {
-            waitpid($pid, 0);
-        } else {
-            while ($pid > 0) {
-                $pid = waitpid($pid, WNOHANG);
-                Coro::AnyEvent::sleep(0.1)
-                    if $pid > 0;
-            }
+        while ($pid > 0) {
+            $pid = waitpid($pid, WNOHANG);
+            Coro::AnyEvent::sleep(0.1)
+              if $pid > 0;
         }
     }
 
@@ -136,7 +125,6 @@ sub send {
 
 sub recv {
     my ($self, $msgid) = @_;
-    croak 'not running' unless $self->is_running;
     my $data = $self->{mailbox}->recv($msgid);
     ++$self->{processed};
 
